import { IWidget } from "./widget";
/**
 * A widget that contains other widgets in a horizontal row
 *
 * Widgets will be laid out next to each other
 */
export declare class Row implements IWidget {
    readonly width: number;
    readonly height: number;
    /**
     * List of contained widgets
     */
    private readonly widgets;
    /**
     * Relative position of each widget inside this row
     */
    private readonly offsets;
    constructor(...widgets: IWidget[]);
    position(x: number, y: number): void;
    toJson(): any[];
}
/**
 * A widget that contains other widgets in a vertical column
 *
 * Widgets will be laid out next to each other
 */
export declare class Column implements IWidget {
    readonly width: number;
    readonly height: number;
    /**
     * List of contained widgets
     */
    private readonly widgets;
    constructor(...widgets: IWidget[]);
    position(x: number, y: number): void;
    toJson(): any[];
}
/**
 * Props of the spacer
 */
export interface SpacerProps {
    /**
     * Width of the spacer
     *
     * @default 1
     */
    readonly width?: number;
    /**
     * Height of the spacer
     *
     * @default: 1
     */
    readonly height?: number;
}
/**
 * A widget that doesn't display anything but takes up space
 */
export declare class Spacer implements IWidget {
    readonly width: number;
    readonly height: number;
    constructor(props: SpacerProps);
    position(_x: number, _y: number): void;
    toJson(): any[];
}
